#ifndef Gradient_HPP
#define Gradient_HPP

#include <cmath>
#include <iostream>
using namespace std;

class gradient {
  double * X;
  int size;
  double w;
  double b;
  double (*phi)(double);
  double (*d_phi)(double);
public:
  gradient (){
    X=nullptr;
    size=0;
    double w=0;
    double b=0;
    phi=nullptr;
    d_phi=nullptr;
  }
  gradient (int n,double (*loss)(double), double (*d_loss)(double)){
    X= new double[n];
    double w=0;
    double b=0;
    size=n;
    phi=loss;
    d_phi=d_loss;
  }
  gradient (const gradient & x){
    phi=x.phi;
    d_phi=x.d_phi;
    w=x.w;
    b=x.b;
    if (x.X!=nullptr){
      X= new double[size];
      for(int i=0;i<size;i++){
        X[i]=x.X[i];
      } 
    }
  }
  ~gradient(){
    if(X != nullptr){
      delete [] X;
    }
  }

  void set_X(int,double);
  double get_w();
  double get_b();
  void compute_gradient(double &,double & );
  void iterate_gradient(int,double);
};

void gradient::compute_gradient(double & dw,double & db ){
  dw=0.0; db=0.0;
  for (int i=0;i<size;i++){
    dw= dw+this->d_phi(w*X[i]+b)*X[i];
    db= db+this->d_phi(w*X[i]+b);
  }
}

void gradient::iterate_gradient(int nmax,double eta){
  double dw,db;
  for (int i=0;i<nmax;i++){
    compute_gradient(dw,db);
    w=w-eta*dw;
    b=b-eta*dw;
  }
}

void gradient::set_X(int i,double x){
  X[i]=x;
}
double gradient::get_w(){
  return w;
}
double gradient::get_b(){
  return b;
}

#endif
